<?php

namespace TAV3V2\Models;

use TAV3V2\Abstracts\Abstract_Main_Plugin_Class;

use TAV3V2\Interfaces\Model_Interface;
use TAV3V2\Interfaces\Activatable_Interface;
use TAV3V2\Interfaces\Initiable_Interface;

use TAV3V2\Helpers\Plugin_Constants;
use TAV3V2\Helpers\Helper_Functions;

/**
 * Model that houses the logic of migration of old versions of TA to version 1.0.0
 *
 * @since 1.0.0
 */
class Migration implements Model_Interface , Initiable_Interface {

    /*
    |--------------------------------------------------------------------------
    | Class Properties
    |--------------------------------------------------------------------------
    */

    /**
     * Property that holds the single main instance of Migration.
     *
     * @since 1.0.0
     * @access private
     * @var Redirection
     */
    private static $_instance;

    /**
     * Model that houses the main plugin object.
     *
     * @since 1.0.0
     * @access private
     * @var Redirection
     */
    private $_main_plugin;

    /**
     * Model that houses all the plugin constants.
     *
     * @since 1.0.0
     * @access private
     * @var Plugin_Constants
     */
    private $_constants;

    /**
     * Property that houses all the helper functions of the plugin.
     *
     * @since 1.0.0
     * @access private
     * @var Helper_Functions
     */
    private $_helper_functions;

    /**
     * Property that holds the list of all affiliate links.
     *
     * @since 1.0.0
     * @access private
     * @var array
     */
    private $_all_affiliate_links;

    /**
     * Variable that holds the mapping between options from old version of the plugin to the new version of the plugin.
     *
     * @since 1.0.0
     * @access public
     * @var array
     */
    private $_new_old_options_mapping;

    /**
     * Variable that holds the mapping between post meta from old version of the plugin to the new version of the plugin.
     *
     * @since 1.0.0
     * @access public
     * @var array
     */
    private $_new_old_meta_mapping;

    /**
     * Property that holds the mapping of per link that have counterpart global options.
     * Well only have a list in the context of version 2 of ta. So new options introduced in v3 ex. uncloak link is not included.
     *
     * @since 1.0.0
     * @access public
     */
    private $_link_meta_global_option_mapping;




    /*
    |--------------------------------------------------------------------------
    | Class Methods
    |--------------------------------------------------------------------------
    */

    /**
     * Class constructor.
     *
     * @since 1.0.0
     * @access public
     *
     * @param Abstract_Main_Plugin_Class $main_plugin      Main plugin object.
     * @param Plugin_Constants           $constants        Plugin constants object.
     * @param Helper_Functions           $helper_functions Helper functions object.
     */
    public function __construct( Abstract_Main_Plugin_Class $main_plugin , Plugin_Constants $constants , Helper_Functions $helper_functions ) {

        $this->_constants           = $constants;
        $this->_helper_functions    = $helper_functions;

        $main_plugin->add_to_all_plugin_models( $this );

    }

    /**
     * Ensure that only one instance of this class is loaded or can be loaded ( Singleton Pattern ).
     *
     * @since 1.0.0
     * @access public
     *
     * @param Abstract_Main_Plugin_Class $main_plugin      Main plugin object.
     * @param Plugin_Constants           $constants        Plugin constants object.
     * @param Helper_Functions           $helper_functions Helper functions object.
     * @return Redirection
     */
    public static function get_instance( Abstract_Main_Plugin_Class $main_plugin , Plugin_Constants $constants , Helper_Functions $helper_functions ) {

        if ( !self::$_instance instanceof self )
            self::$_instance = new self( $main_plugin , $constants , $helper_functions );

        return self::$_instance;

    }

    /**
     * Hide migration tools if a data downgrade is currently running.
     *
     * @since 1.0.0
     * @access public
     *
     * @return boolean False to prevent showing migration tools.
     */
    public function hide_migration_tools() {

        if ( Plugin_Constants::DOWNGRADE_COMPLETE_FLAG === 'no' )
            return false;
        else
            return true;

    }

    /**
     * Initialize data key mappings.
     *
     * @since 1.0.0
     * @access public
     */
    public function initialize_data_key_mappings() {

        // Get all affiliate links
        $this->_all_affiliate_links = $this->get_all_affiliate_links();

        // TODO: Currently does not include mapping for options added via addon. Inclue this.
        $this->_new_old_options_mapping = apply_filters( 'ta_new_old_options_mapping' , array(
            'ta_link_prefix'                      => 'linkprefix',
            'ta_link_prefix_custom'               => 'linkprefixcustom',
            'ta_show_cat_in_slug'                 => 'showcatinslug',
            'ta_disable_cat_auto_select'          => 'disablecatautoselect',
            'ta_link_redirect_type'               => 'linkredirecttype',
            'ta_no_follow'                        => 'nofollow',
            'ta_new_window'                       => 'newwindow',
            // 'legacyuploader' : Not touched on TA V3
            'ta_disable_title_attribute'          => 'disabletitleattribute',
            'ta_disable_thirsty_link_class'       => 'disablethirstylinkclass',
            'ta_disable_slug_shortening'          => 'disableslugshortening',
            'ta_disable_visual_editor_buttons'    => 'disablevisualeditorbuttons',
            'ta_disable_text_editor_buttons'      => 'disabletexteditorbuttons',
            'ta_additional_rel_tags'              => 'additionalreltags',

            /* Autolinker */
            'tap_autolink_bbpress'                => 'autolinkbbpress',
            'tap_autolink_random_placement'       => 'randomplacement',
            'tap_autolink_inside_heading'         => 'autolinkheadings',
            'tap_autolink_disable_archives'       => 'disablearchives',
            'tap_autolink_disable_homepage'       => 'disablehome',
            'tap_autolink_enable_feeds'           => 'enablefeedreplacement',
            'tap_autolink_post_types'             => 'enabledPostTypes',

            /* Geolocations */
            'tap_geolocations_disable_proxy_test' => 'disableforwardingproxytest',
            // 'tap_geolocations_enable_maxmind'     => 'enablemaxminddb',
            'tap_geolocations_maxmind_mmdb_file'  => 'maxminddbfile',
            // 'tap_geolocations_enable_maxmind_web' => 'enablemaxmindwebservice',
            'tap_geolocations_maxmind_api_userid' => 'maxmindwebserviceuserid',
            'tap_geolocations_maxmind_api_key'    => 'maxmindwebservicekey',

            // Google Click Tracking
            'tap_google_click_tracking_action_name'   => 'gctactionname',
            'tap_google_click_tracking_use_post'      => 'gctusepost',
            'tap_google_click_tracking_use_legacy_ga' => 'gctuselegacyga',
            // 'gctfilterlegacyga' : Not touched on TA V3

            /* Amazon */
            'tap_amazon_access_key_id'           => 'azon_aws_access_key_id',
            'tap_amazon_secret_key'              => 'azon_aws_secret_key',
            'tap_last_used_search_endpoint'      => 'defaultSearchCountry',
            'tap_azon_geolocation_integration'   => 'azon_geolocation_support',
            'tap_azon_imported_link_categories'  => 'defaultCategories',
            'tap_hide_products_with_empty_price' => 'exclude_zero_priced_products',

            /* Stats */
            'tap_stats_manual_exclusions' => 'statsmanualexclusions',

        ) );

        // TODO: Currently does not include mapping for post meta added via addon. Include this.
        $this->_new_old_meta_mapping = apply_filters( 'ta_new_old_meta_mapping' , array(
            'redirect_type'             => 'linkredirecttype',
            'destination_url'           => 'linkurl',
            'no_follow'                 => 'nofollow',
            'new_window'                => 'newwindow',
            // 'enablewildcard' : Tentative, Not touched on TA V3
            // 'wildcards'      : Tentative, Not touched on TA V3
            'name'                      => 'linkname',

            /* Autolinker */
            'autolink_keyword_list'     => 'keywordlist',
            'autolink_keyword_limit'    => 'autolinklimit',
            'autolink_inside_heading'   => 'autolinkheadings',
            'autolink_random_placement' => 'randomplacement',

            /* Geolocations */
            'geolocation_links'         => 'geolink',

            /* Amazon */
            '_tap_asin'                 => 'asin'
        ) );

        $this->_link_meta_global_option_mapping = apply_filters( 'ta_link_meta_global_option_mapping' , array(
            'no_follow'                 => 'ta_no_follow',
            'new_window'                => 'ta_new_window',
            'autolink_inside_heading'   => 'tap_autolink_inside_heading',
            'autolink_random_placement' => 'tap_autolink_random_placement'
        ) );

    }

    /**
     * Migrate old plugin data.
     * Note: Plugin_Constants::DOWNGRADE_COMPLETE_FLAG is not used to determine if we should downgrade or not
     * The only purpose of this option is to determine if there is a current downgrade running.
     * If it says 'no' then there is a downgrade running, but its not yet finished.
     * If it says 'yes' then it says, the current downgrade is done.
     * We need to allow re-running downgrade coz there might be some data that have not downgraded on first pass ( though this is for worst case scenarios ).
     *
     * @since 1.0.0
     * @access public
     */
    public function downgrade_v3_plugin_data() {

        update_option( Plugin_Constants::DOWNGRADE_COMPLETE_FLAG , 'no' );

        $this->initialize_data_key_mappings();
        $this->downgrade_link_meta();
        $this->downgrade_plugin_options();
        $this->downgrade_image_attachments();

        do_action( 'tav3v2_downgrade_plugin_data' );

        update_option( Plugin_Constants::DOWNGRADE_COMPLETE_FLAG , 'yes' );

    }

    /**
     * Migrate old plugin data to new data model via ajax.
     *
     * @since 1.0.0
     * @access public
     */
    public function ajax_downgrade_v3_plugin_data() {

        if ( ! defined( 'DOING_AJAX' ) || ! DOING_AJAX )
            $response = array( 'status' => 'fail' , 'error_msg' => __( 'Invalid AJAX Call.' , 'tav3v2' ) );
        elseif ( !$this->_helper_functions->current_user_authorized() )
            $response = array( 'status' => 'fail' , 'error_msg' => __(  'Unauthorized operation. Only authorized accounts can downgrade data.' , 'tav3v2' ) );
        else {

            $this->downgrade_v3_plugin_data();
            $response = array( 'status' => 'success' , 'success_msg' => __( 'Data Downgrade Complete!' , 'tav3v2' ) );

        }

        @header( 'Content-Type: application/json; charset=' . get_option( 'blog_charset' ) );
        echo wp_json_encode( $response );
        wp_die();

    }




    /*
    |--------------------------------------------------------------------------
    | Migrating Settings Options
    |--------------------------------------------------------------------------
    */

    /**
     * Downgrade v3 plugin data to v2.
     *
     * @since 1.0.0
     * @access public
     */
    public function downgrade_plugin_options() {

        $old_options = get_option( 'thirstyOptions' , array() );
        if ( !is_array( $old_options ) )
            $old_options = array();

        foreach ( $this->_new_old_options_mapping as $new_key => $old_key ) {

            if ( array_key_exists( $old_key , $old_options ) )
                continue;

            $new_option_val = get_option( $new_key );

            if ( $new_option_val === 'no' || empty( $new_option_val ) ) {

                delete_option( $new_key );
                continue;

            } elseif ( $new_option_val === 'yes' )
                $old_options[ $old_key ] = 'on';
            else
                $old_options[ $old_key ] = $new_option_val;

            delete_option( $new_key );

        }

        $old_options = apply_filters( 'tav3v2_downgrade_plugin_options' , $old_options );

        update_option( 'thirstyOptions' , $old_options );

        do_action( 'tav3v2_plugin_option_downgrade' ); // Hook for downgrading complex options

    }

    /**
     * Downgrade complex options.
     *
     * @since 1.0.0
     * @access public
     */
    public function downgrade_complex_options() {

        $old_options = get_option( 'thirstyOptions' , array() );
        if ( !is_array( $old_options ) )
            $old_options = array();

        // Downgrade Amazon Associate Tags
        $new_associate_tags = get_option( 'tap_amazon_associate_tags' );
        if ( !is_array( $new_associate_tags ) )
            $new_associate_tags = array();

        $country_codes      = array( 'us' , 'ca' , 'cn' , 'de' , 'es' , 'fr' , 'in' , 'it' , 'jp' , 'uk' ); // The only supported countries on old azon

        foreach ( $country_codes as $cc ) {

            if ( array_key_exists( strtoupper( $cc ) , $new_associate_tags ) )
                $old_options[ $cc . "_azon_aws_associate_tag" ] = $new_associate_tags[ strtoupper( $cc ) ];
            else
                $old_options[ $cc . "_azon_aws_associate_tag" ] = "";

        }

        delete_option( 'tap_amazon_associate_tags' );

        // Downgrade Import Images
        $imported_images = get_option( 'tap_azon_import_images' );

        if ( !empty( $imported_images ) && is_array( $imported_images ) ) {

            $old_options[ 'importImages' ] = array();

            foreach ( $imported_images as $img )
                $old_options[ 'importImages' ][ $img ] = 'on';

        }

        delete_option( 'tap_azon_import_images' );

        // Update the old option so no repeating of imports
        update_option( 'thirstyOptions' , $old_options );

    }




    /*
    |--------------------------------------------------------------------------
    | Migrating Link Meta
    |--------------------------------------------------------------------------
    */

    /**
     * Get all affiliate links.
     *
     * @since 1.0.0
     * @access public
     *
     * @return array Array of all affiliate links as objects.
     */
    public function get_all_affiliate_links() {

        global $wpdb;

        $query = "SELECT *
                  FROM $wpdb->posts
                  WHERE post_type = 'thirstylink'";

        return $wpdb->get_results( $query );

    }

    /**
     * Downgrade v3 plugin data down to v2.
     *
     * @since 1.0.0
     * @access public
     *
     * @global WPDB $wpdb Global $wpdb object.
     */
    public function downgrade_link_meta() {

        global $wpdb;

        foreach ( $this->_all_affiliate_links as $affiliate_link ) {

            $old_link_meta = maybe_unserialize( get_post_meta( $affiliate_link->ID , 'thirstyData' , true ) );
            if ( !is_array( $old_link_meta ) )
                $old_link_meta = array();

            // set linkname meta
            $old_link_meta[ 'linkname' ] = get_the_title( $affiliate_link->ID );

            foreach ( $this->_new_old_meta_mapping as $new_key => $old_key ) {

                if ( array_key_exists( $old_key , $old_link_meta ) )
                    continue;

                $meta_value = get_post_meta( $affiliate_link->ID , Plugin_Constants::META_DATA_PREFIX . $new_key , true );

                if ( array_key_exists( $new_key , $this->_link_meta_global_option_mapping ) ) {

                    // get global value if meta is set to 'global'
                    if ( $meta_value === 'global' )
                        $meta_value = get_option( $this->_link_meta_global_option_mapping[ $new_key ] );

                    $meta_value = $meta_value == 'yes' ? 'on' : $meta_value;

                    // if value is not set to "on", then unset on $old_link_meta and skip.
                    if ( $meta_value !== 'on' ) {

                        unset( $old_link_meta[ $old_key ] );
                        delete_post_meta( $affiliate_link->ID , Plugin_Constants::META_DATA_PREFIX . $new_key );
                        continue;
                    }
                }

                $old_link_meta[ $old_key ] = $meta_value === 'yes' ? 'on' : $meta_value;
                delete_post_meta( $affiliate_link->ID , Plugin_Constants::META_DATA_PREFIX . $new_key );

            }

            $old_link_meta = apply_filters( 'tav3v2_downgrade_link_meta' , $old_link_meta , $affiliate_link );

            update_post_meta( $affiliate_link->ID , 'thirstyData' , serialize( $old_link_meta ) );

        }

    }




    /*
    |--------------------------------------------------------------------------
    | Migrating Image Attachments To An Affiliate Link
    |--------------------------------------------------------------------------
    */

    /**
     * Downgrade image attachments data to v2.
     *
     * @since 1.0.0
     * @access public
     *
     * @global WPDB $wpdb Global $wpdb object.
     */
    public function downgrade_image_attachments() {

        global $wpdb;

        foreach ( $this->_all_affiliate_links as $affiliate_link ) {

            $attached_images_ids = get_post_meta( $affiliate_link->ID , Plugin_Constants::META_DATA_PREFIX . 'image_ids' , true );
            if ( !is_array( $attached_images_ids ) )
                $attached_images_ids = array();

            if ( !empty( $attached_images_ids ) ) {

                $wpdb->get_results( "UPDATE $wpdb->posts
                                     SET post_parent = $affiliate_link->ID
                                     WHERE post_type = 'attachment'
                                     AND ID IN ( " . implode( "," , array_map( 'intval' , $attached_images_ids ) ) . " )" );

                delete_post_meta( $affiliate_link->ID , Plugin_Constants::META_DATA_PREFIX . 'image_ids' );

            }

        }

    }

    /**
     * Downgrade geolocations metadata
     *
     * @since 1.0.0
     * @access public
     *
     * @param array $old_link_data Affiliate link old post meta data.
     * @param object $affiliate_link Affiliate link object $wpdb->posts single row result.
     * @return array Filtered affiliate link old post meta data.
     */
     public function downgrade_geolocations_meta_data( $old_link_meta , $affiliate_link ) {

         if ( empty( $old_link_meta ) || ! isset( $old_link_meta[ 'geolink' ] ) || ! is_array( $old_link_meta[ 'geolink' ] ) || empty( $old_link_meta[ 'geolink' ] ) )
            return $old_link_meta;

        $old_geolinks = array();

        foreach ( $old_link_meta[ 'geolink' ] as $key => $destination_url ) {

            $countries     = explode( ',' , $key );
            $count         = 1;
            $first_country = '';

            foreach ( $countries as $country ) {

                if ( $count == 1 ) {

                    $old_geolinks[ $country ] = $destination_url;
                    $first_country            = $country;

                } else
                    $old_geolinks[ $country ] = $first_country;

                $count++;
            }
        }

        $old_link_meta[ 'geolink' ] = $old_geolinks;

        return $old_link_meta;
    }

    /**
     * Downgrade autolinker post type selection fields.
     *
     * @since 1.0.0
     * @access public
     *
     * @param array $old_options List of TA2 old options data.
     * @return array Filtered List of TA2 old options data.
     */
    public function downgrade_autolinker_post_types( $old_options ) {

        if ( isset( $old_options[ 'enabledPostTypes' ] ) && ! empty( $old_options[ 'enabledPostTypes' ] ) ) {

            $old_post_types = array();

            foreach ( $old_options[ 'enabledPostTypes' ] as $post_type )
                $old_post_types[ $post_type ] = 'on';

            $old_options[ 'enabledPostTypes' ] = $old_post_types;
        }

        return $old_options;
    }

    /**
     * Downgrade stats manual exclusion while adding back the default value added by the Stats addon.
     *
     * @since 1.0.0
     * @access public
     *
     * @param array $old_options List of TA2 old options data.
     * @return array Filtered List of TA2 old options data.
     */
    public function downgrade_stats_manual_exclusion( $old_options ) {

        if ( ! isset( $old_options[ 'statsmanualexclusions' ] ) )
            return $old_options;

        $default_exclusions = "64.233.160.0-64.233.191.255\r\n66.102.0.0-66.102.15.255\r\n66.249.64.0-66.249.95.255\r\n72.14.192.0-72.14.255.255\r\n74.125.0.0-74.125.255.255\r\n209.85.128.0-209.85.255.255\r\n216.239.32.0-216.239.63.255\r\n64.4.0.0-64.4.63.255\r\n65.52.0.0-65.55.255.255\r\n131.253.21.0-131.253.47.255\r\n157.54.0.0-157.60.255.255\r\n207.46.0.0-207.46.255.255\r\n207.68.128.0-207.68.207.255\r\n8.12.144.0-8.12.144.255\r\n66.196.64.0-66.196.127.255\r\n66.228.160.0-66.228.191.255\r\n67.195.0.0-67.195.255.255\r\n68.142.192.0-68.142.255.255\r\n72.30.0.0-72.30.255.255\r\n74.6.0.0-74.6.255.255\r\n98.136.0.0-98.139.255.255\r\n202.160.176.0-202.160.191.255\r\n209.191.64.0-209.191.127.255";
        $old_options[ 'statsmanualexclusions' ] = $default_exclusions . "\r\n" . $old_options[ 'statsmanualexclusions' ];

        return $old_options;
    }

    /**
     * Make sure that autolinker options are removed in the `thirstyOptions` serialized data when they are not turned on
     * in the settings. This is due to a bug in the old Autolinker addon, where the checkbox fields are only checked with isset().
     *
     * @since 1.0.0
     * @access public
     */
    public function autolinker_settings_fixer() {

        $old_options = get_option( 'thirstyOptions' , array() );
        if ( !is_array( $old_options ) )
            $old_options = array();

        $autolinker_options = array(
            'autolinkbbpress',
            'randomplacement',
            'autolinkheadings',
            'disablearchives',
            'disablehome',
            'enablefeedreplacement'
        );

        foreach ( $autolinker_options as $option ) {

            if (  isset( $old_options[ $option ] ) && $old_options[ $option ] != 'on' )
                unset( $old_options[ $option ] );
        }

        update_option( 'thirstyOptions' , $old_options );

    }

    /**
     * Downgrade new GCT tracking script option. (Added on TAP 1.1.0)
     *
     * @since 1.0.1
     * @access public
     *
     * @param array $old_options List of TA2 old options data.
     * @return array Filtered List of TA2 old options data.
     */
    public function downgrade_gct_tracking_script( $old_options ) {

        if ( ! is_plugin_active( 'thirstyaffiliates-pro/thirstyaffiliates-pro.php' ) )
            return $old_options;

        $tap_plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/thirstyaffiliates-pro/thirstyaffiliates-pro.php' );

        if ( version_compare( $tap_plugin_data[ 'Version' ] , '1.1.0' , "<" ) )
            return $old_options;

        $tracking_script = get_option( 'tap_google_click_tracking_script' , 'universal_ga' );

        delete_option( 'tap_google_click_tracking_script' );

        if ( $tracking_script == 'legacy_ga' )
            $old_options[ 'gctuselegacyga' ] = 'on';
        else
            unset( $old_options[ 'gctuselegacyga' ] );

        return $old_options;
    }




    /*
    |--------------------------------------------------------------------------
    | Migration Admin Notice
    |--------------------------------------------------------------------------
    */

    /**
     *  Show admin notice that downgrade process is currently running.
     *
     * @since 1.0.0
     * @access public
     */
    public function downgrade_running_admin_notice() {

        if ( get_option( Plugin_Constants::DOWNGRADE_COMPLETE_FLAG ) === 'no' ) { ?>

            <div class="notice notice-warning">
                <p><?php _e( '<b>We are currently downgrading your ThirstyAffiliates data from version 3 down to version 2.<br>Please wait until this process finishes before making any changes to your affiliate links.<b>' , 'tav3v2' ); ?></p>
            </div>

        <?php }

    }




    /*
    |--------------------------------------------------------------------------
    | Fulfill Implemented Interface Contracts
    |--------------------------------------------------------------------------
    */

    /**
     * Execute codes that needs to run on plugin initialization.
     *
     * @since 1.0.0
     * @access public
     * @implements TAV3V2\Interfaces\Initiable_Interface
     */
    public function initialize() {

        add_action( 'wp_ajax_ta_downgrade_v3_plugin_data' , array( $this , 'ajax_downgrade_v3_plugin_data' ) );
        add_filter( 'ta_database_processing' , array( $this , 'hide_migration_tools' ) , 10 );

    }

    /**
     * Execute Migration class.
     *
     * @since 1.0.0
     * @access public
     * @implements TAV3V2\Interfaces\Model_Interface
     */
    public function run() {

        add_action( 'admin_notices' , array( $this , 'downgrade_running_admin_notice' ) );
        add_filter( 'tav3v2_downgrade_link_meta' , array( $this , 'downgrade_geolocations_meta_data' ) , 10 , 2 );
        add_filter( 'tav3v2_downgrade_plugin_options' , array( $this , 'downgrade_autolinker_post_types' ) , 10 );
        add_filter( 'tav3v2_downgrade_plugin_options' , array( $this , 'downgrade_stats_manual_exclusion' ) , 10 );
        add_filter( 'tav3v2_downgrade_plugin_options' , array( $this , 'downgrade_gct_tracking_script' ) , 10 );

        add_action( 'tav3v2_plugin_option_downgrade' , array( $this , 'downgrade_complex_options' ) );
        add_action( 'tav3v2_plugin_option_downgrade' , array( $this , 'autolinker_settings_fixer' ) );

    }

}
