<?php
namespace TAV3V2\Models;

use TAV3V2\Abstracts\Abstract_Main_Plugin_Class;

use TAV3V2\Interfaces\Model_Interface;

use TAV3V2\Helpers\Plugin_Constants;
use TAV3V2\Helpers\Helper_Functions;

if ( !defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * The purpose of this model is to show an example on how to Extend the WPB Settings API.
 * Of course the realistic use case external plugins extending this plugin's Settings API.
 * We just add this file here for illustration purposes.
 * Please delete this file when creating a plugin from this boilerplate.
 *
 * Private Model.
 *
 * @since 1.0.0
 */
class Settings_Extension implements Model_Interface {

    /*
    |--------------------------------------------------------------------------
    | Class Properties
    |--------------------------------------------------------------------------
    */

    /**
     * Property that holds the single main instance of Settings_Extension.
     *
     * @since 1.0.0
     * @access private
     * @var Settings_Extension
     */
    private static $_instance;

    /**
     * Model that houses all the plugin constants.
     *
     * @since 1.0.0
     * @access private
     * @var Plugin_Constants
     */
    private $_constants;

    /**
     * Property that houses all the helper functions of the plugin.
     *
     * @since 1.0.0
     * @access private
     * @var Helper_Functions
     */
    private $_helper_functions;




    /*
    |--------------------------------------------------------------------------
    | Class Methods
    |--------------------------------------------------------------------------
    */

    /**
     * Class constructor.
     *
     * @since 1.0.0
     * @access public
     *
     * @param Abstract_Main_Plugin_Class $main_plugin      Main plugin object.
     * @param Plugin_Constants           $constants        Plugin constants object.
     * @param Helper_Functions           $helper_functions Helper functions object.
     */
    public function __construct( Abstract_Main_Plugin_Class $main_plugin , Plugin_Constants $constants , Helper_Functions $helper_functions ) {

        $this->_constants        = $constants;
        $this->_helper_functions = $helper_functions;

        $main_plugin->add_to_all_plugin_models( $this );

    }

    /**
     * Ensure that only one instance of this class is loaded or can be loaded ( Singleton Pattern ).
     *
     * @since 1.0.0
     * @access public
     *
     * @param Abstract_Main_Plugin_Class $main_plugin      Main plugin object.
     * @param Plugin_Constants           $constants        Plugin constants object.
     * @param Helper_Functions           $helper_functions Helper functions object.
     * @return Settings_Extension
     */
    public static function get_instance( Abstract_Main_Plugin_Class $main_plugin , Plugin_Constants $constants , Helper_Functions $helper_functions ) {

        if ( !self::$_instance instanceof self )
            self::$_instance = new self( $main_plugin , $constants , $helper_functions );

        return self::$_instance;

    }

    /**
     * Register the rebuild cache button for the Autolinker cache.
     *
     * @since 1.0.0
     * @access public
     *
     * @param array $supported_field_types Array of all supported field types for the WPB Settings API.
     * @return array Filtered array of supported field types.
     */
    public function register_downgrade_data_field( $supported_field_types ) {

        if ( array_key_exists( 'downgrade_data' , $supported_field_types ) )
            return $supported_field_types;

        $supported_field_types[ 'downgrade_data' ] = function( $option ) {

            // This function will be the render callback of this new custom field type
            // It will expect 1 parameter to be passed by our Settings API, and that is the $option data

            $processing = "";
            $disabled   = false;

            if ( get_option( Plugin_Constants::DOWNGRADE_COMPLETE_FLAG ) === 'no' ) {

                $processing = "-processing";
                $disabled   = true;

            } ?>

            <tr class="<?php echo $option[ 'id' ] . '-row'; ?>">
                <th scope="row"><?php echo $option[ 'title' ]; ?></th>
                <td class="forminp forminp-<?php echo sanitize_title( $option[ 'type' ] ) ?> <?php echo $processing; ?>">

                    <?php if ( get_option( Plugin_Constants::MIGRATION_COMPLETE_FLAG ) === 'no' ) { ?>

                        <p><?php _e( 'Another application is currently processing the database. Please wait for this to complete.' , 'tav3v2' ); ?></p>

                    <?php } else { ?>

                        <input
                            <?php echo $disabled ? "disabled" : ""; ?>
                            type="button"
                            id="<?php echo esc_attr( $option[ 'id' ] ); ?>"
                            class="button button-primary"
                            style="<?php echo isset( $option[ 'css' ] ) ? esc_attr( $option[ 'css' ] ) : ''; ?>"
                            value="<?php _e( 'Downgrade' , 'tav3v2' ); ?>">

                        <span class="spinner"></span>
                        <p class="desc"><?php _e( 'Downgrading data. Please wait...' , 'tav3v2' ); ?></p>

                    <?php } ?>

                </td>
            </tr>
            <?php

        };

        return $supported_field_types;
    }

    /**
     * Register a new custom settings section ( TAV3V2 ).
     *
     * @since 1.0.0
     * @access public
     *
     * @param array $settings_sections All registered settings sections.
     * @return array Filtered registered settings sections.
     */
    public function register_tav3v2_settings_section( $settings_sections ) {

        // register autolinker settings
        if ( ! array_key_exists( 'tav3v2_settings' , $settings_sections ) ) {
            $settings_sections[ 'tav3v2_settings' ] = array(
                'title' => __( 'Downgrade' , 'tav3v2' ),
                'desc'  => __( 'Downgrade ThirstyAffiliates version 3 data to version 2' , 'tav3v2' )
            );
        }

        return $settings_sections;
    }

    /**
     * Register options for the TAV3V2 module settings section.
     *
     * @since 1.0.0
     * @access public
     *
     * @param array $settings_section_options Registered options for each registered settings section.
     * @return array Filtered registered options for each registered settings section.
     */
    public function register_tav3v2_settings_options( $settings_section_options ) {

        if ( array_key_exists( 'tav3v2_settings' , $settings_section_options ) )
            return;

        $settings_section_options[ 'tav3v2_settings' ] = apply_filters(
            'tav3v2_settings_option' , // Add a filter to the array of options for this section so that others can extend it
            array(
                array(
                    'id'    => 'ta_downgrade_v3_data_to_v2',
                    'title' => __( 'Downgrade Data' , 'tav3v2' ),
                    'type'  => 'downgrade_data'
                )
            )
        );

        return $settings_section_options;
    }




    /*
    |--------------------------------------------------------------------------
    | Implemented Interface Methods
    |--------------------------------------------------------------------------
    */

    /**
     * Execute model.
     *
     * @implements ThirstyAffiliates_Pro\Interfaces\Model_Interface
     *
     * @since 1.0.0
     * @access public
     */
    public function run() {

        // Register your custom field types here
        // It is advisable to register them separately, not registering all your custom field types in one function ( S in SOLID principle )
        add_filter( 'ta_supported_field_types' , array( $this , 'register_downgrade_data_field' ) );

        // Register your custom settings section here
        add_filter( 'ta_settings_option_sections' , array( $this , 'register_tav3v2_settings_section' ) );

        // Register the set of options to the new custom settings section you've added above
        // It is advisable to register them separately, not registering all your custom settings sections in one function ( S in SOLID principle )
        add_filter( 'ta_settings_section_options' , array( $this , 'register_tav3v2_settings_options' ) );
    }

}
